# coding: utf-8

"""
    Payment API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self


class MinAmountReq(BaseModel):
    """
    Get the minimum payment amount for a specific currency pair. https://documenter.getpostman.com/view/7907941/2s93JusNJt#ce3fe3a3-00cd-4df2-bfba-641fde741da7  You can provide both currencies in the pair or just currency_from. If only currency_from is provided, the minimum payment amount will be calculated for currency_from and the default currency specified in Payment Settings.  You can specify a fiat currency in currency_from to calculate the minimum payment in that fiat currency.  The fiat_equivalent field can be added to get the fiat equivalent of the minimum amount.  The is_fixed_rate and is_fee_paid_by_user parameters allow you to see current minimal amounts for corresponding flows, which may differ from the standard flow.  For multiple outcome wallets, the minimum amount is calculated based on the payment routing logic.
    """  # noqa: E501

    currency_from: StrictStr = Field(description="Payin currency")
    currency_to: Optional[StrictStr] = None
    fiat_equivalent: Optional[StrictStr] = None
    is_fixed_rate: Optional[StrictBool] = None
    is_fee_paid_by_user: Optional[StrictBool] = None
    __properties: ClassVar[List[str]] = [
        "currency_from",
        "currency_to",
        "fiat_equivalent",
        "is_fixed_rate",
        "is_fee_paid_by_user",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of MinAmountReq from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if currency_to (nullable) is None
        # and model_fields_set contains the field
        if self.currency_to is None and "currency_to" in self.model_fields_set:
            _dict["currency_to"] = None

        # set to None if fiat_equivalent (nullable) is None
        # and model_fields_set contains the field
        if self.fiat_equivalent is None and "fiat_equivalent" in self.model_fields_set:
            _dict["fiat_equivalent"] = None

        # set to None if is_fixed_rate (nullable) is None
        # and model_fields_set contains the field
        if self.is_fixed_rate is None and "is_fixed_rate" in self.model_fields_set:
            _dict["is_fixed_rate"] = None

        # set to None if is_fee_paid_by_user (nullable) is None
        # and model_fields_set contains the field
        if (
            self.is_fee_paid_by_user is None
            and "is_fee_paid_by_user" in self.model_fields_set
        ):
            _dict["is_fee_paid_by_user"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of MinAmountReq from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "currency_from": obj.get("currency_from"),
                "currency_to": obj.get("currency_to"),
                "fiat_equivalent": obj.get("fiat_equivalent"),
                "is_fixed_rate": obj.get("is_fixed_rate"),
                "is_fee_paid_by_user": obj.get("is_fee_paid_by_user"),
            }
        )
        return _obj



from typing import Optional, Callable, Any, Annotated
from typing_extensions import Doc
from enum import Enum

from fastapi import params
from fastapi.security import APIKeyHeader, HTTPBearer

from crypticorn.common import Scope


http_bearer = HTTPBearer(
    bearerFormat="JWT",
    auto_error=False,
    description="The JWT to use for authentication.",
)

apikey_header = APIKeyHeader(
    name="X-API-Key",
    auto_error=False,
    description="The API key to use for authentication.",
)


def Security(  # noqa: N802
    dependency: Annotated[
        Optional[Callable[..., Any]], Doc("A dependable callable (like a function).")
    ] = None,
    scopes: Annotated[
        Optional[list[Scope]],
        Doc("API scopes required for the *path operation*."),
    ] = None,
    use_cache: Annotated[
        bool, Doc("Whether to cache the dependency during a single request.")
    ] = True,
) -> Any:
    """Security Dependency. Extends the fastapi.Security class to support enum scopes instead of strings."""

    # Convert Enum scopes to string
    scopes_str = [s.value for s in scopes] if scopes else None

    return params.Security(
        dependency=dependency, scopes=scopes_str, use_cache=use_cache
    )



import pandas as pd
from crypticorn.klines import (
    ApiClient,
    Configuration,
    FundingRatesApi,
    HealthCheckApi,
    OHLCVDataApi,
    SymbolsApi,
    UDFApi,
)
from crypticorn.common import BaseURL, ApiVersion, Service, apikey_header as aph


class FundingRatesApiWrapper(FundingRatesApi):
    """
    A wrapper for the FundingRatesApi class.
    """

    def get_funding_rates_fmt(self):
        response = self.funding_rate_funding_rates_symbol_get()
        return pd.DataFrame(response.json())


class OHLCVDataApiWrapper(OHLCVDataApi):
    """
    A wrapper for the OHLCVDataApi class.
    """

    def get_ohlcv_data_fmt(self):
        response = self.get_ohlcv_market_timeframe_symbol_get()
        return pd.DataFrame(response.json())


class SymbolsApiWrapper(SymbolsApi):
    """
    A wrapper for the SymbolsApi class.
    """

    def get_symbols_fmt(self):
        response = self.symbols_symbols_market_get()
        return pd.DataFrame(response.json())


class UDFApiWrapper(UDFApi):
    """
    A wrapper for the UDFApi class.
    """

    def get_udf_fmt(self):
        response = self.get_history_udf_history_get()
        return pd.DataFrame(response.json())


class KlinesClient:
    """
    A client for interacting with the Crypticorn Klines API.
    """

    def __init__(
        self,
        base_url: BaseURL,
        api_version: ApiVersion,
        api_key: str = None,
        jwt: str = None,
    ):
        self.host = f"{base_url.value}/{api_version.value}/{Service.KLINES.value}"
        self.config = Configuration(
            host=self.host,
            access_token=jwt,
            api_key={aph.scheme_name: api_key} if api_key else None,
            api_key_prefix=({aph.scheme_name: aph.model.name} if api_key else None),
        )
        self.base_client = ApiClient(configuration=self.config)
        # Instantiate all the endpoint clients
        self.funding = FundingRatesApiWrapper(self.base_client)
        self.ohlcv = OHLCVDataApiWrapper(self.base_client)
        self.symbols = SymbolsApiWrapper(self.base_client)
        self.udf = UDFApiWrapper(self.base_client)
        self.health = HealthCheckApi(self.base_client)



# coding: utf-8

# flake8: noqa

"""
    Hive AI API

    API for Hive AI model training and evaluation

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from crypticorn.hive.client.api.data_api import DataApi
from crypticorn.hive.client.api.models_api import ModelsApi
from crypticorn.hive.client.api.status_api import StatusApi

# import ApiClient
from crypticorn.hive.client.api_response import ApiResponse
from crypticorn.hive.client.api_client import ApiClient
from crypticorn.hive.client.configuration import Configuration
from crypticorn.hive.client.exceptions import OpenApiException
from crypticorn.hive.client.exceptions import ApiTypeError
from crypticorn.hive.client.exceptions import ApiValueError
from crypticorn.hive.client.exceptions import ApiKeyError
from crypticorn.hive.client.exceptions import ApiAttributeError
from crypticorn.hive.client.exceptions import ApiException

# import models into sdk package
from crypticorn.hive.client.models.coins import Coins
from crypticorn.hive.client.models.data_download_response import DataDownloadResponse
from crypticorn.hive.client.models.data_info import DataInfo
from crypticorn.hive.client.models.data_value_value_value_inner import (
    DataValueValueValueInner,
)
from crypticorn.hive.client.models.data_version import DataVersion
from crypticorn.hive.client.models.download_links import DownloadLinks
from crypticorn.hive.client.models.evaluation import Evaluation
from crypticorn.hive.client.models.evaluation_response import EvaluationResponse
from crypticorn.hive.client.models.feature_size import FeatureSize
from crypticorn.hive.client.models.http_validation_error import HTTPValidationError
from crypticorn.hive.client.models.model import Model
from crypticorn.hive.client.models.model_create import ModelCreate
from crypticorn.hive.client.models.model_status import ModelStatus
from crypticorn.hive.client.models.model_update import ModelUpdate
from crypticorn.hive.client.models.target import Target
from crypticorn.hive.client.models.target_type import TargetType
from crypticorn.hive.client.models.validation_error import ValidationError
from crypticorn.hive.client.models.validation_error_loc_inner import (
    ValidationErrorLocInner,
)



from enum import Enum


class Scope(str, Enum):
    """
    The permission scopes for the API.
    """

    # If you update anything here, also update the scopes in the auth-service repository

    @classmethod
    def from_str(cls, value: str) -> "Scope":
        return cls(value)

    # Hive scopes
    READ_HIVE_MODEL = "read:hive:model"
    READ_HIVE_DATA = "read:hive:data"
    WRITE_HIVE_MODEL = "write:hive:model"

    # Trade scopes
    READ_TRADE_BOTS = "read:trade:bots"
    WRITE_TRADE_BOTS = "write:trade:bots"
    READ_TRADE_APIKEYS = "read:trade:api_keys"
    WRITE_TRADE_APIKEYS = "write:trade:api_keys"
    READ_TRADE_ORDERS = "read:trade:orders"
    READ_TRADE_ACTIONS = "read:trade:actions"
    WRITE_TRADE_ACTIONS = "write:trade:actions"
    READ_TRADE_EXCHANGES = "read:trade:exchanges"
    READ_TRADE_FUTURES = "read:trade:futures"
    WRITE_TRADE_FUTURES = "write:trade:futures"
    READ_TRADE_NOTIFICATIONS = "read:trade:notifications"
    WRITE_TRADE_NOTIFICATIONS = "write:trade:notifications"
    READ_TRADE_STRATEGIES = "read:trade:strategies"
    WRITE_TRADE_STRATEGIES = "write:trade:strategies"

    # Payment scopes
    READ_PAY_PAYMENTS = "read:pay:payments"
    READ_PAY_PRODUCTS = "read:pay:products"
    WRITE_PAY_PRODUCTS = "write:pay:products"
    READ_PAY_NOW = "read:pay:now"
    WRITE_PAY_NOW = "write:pay:now"

    # Read projections
    READ_PREDICTIONS = "read:predictions"



# coding: utf-8

"""
    Payment API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Union
from crypticorn.pay.client.models.payment_status import PaymentStatus
from crypticorn.pay.client.models.services import Services
from typing import Optional, Set
from typing_extensions import Self


class UnifiedPaymentModel(BaseModel):
    """
    Combined payment model across all services
    """  # noqa: E501

    id: StrictStr = Field(description="Payment ID")
    product_id: StrictStr = Field(description="Product ID")
    var_date: StrictInt = Field(description="Payment date in seconds", alias="date")
    amount: Union[StrictFloat, StrictInt] = Field(description="Payment amount")
    currency: StrictStr = Field(description="Payment currency")
    status: PaymentStatus
    service: Services = Field(description="Payment service")
    market: StrictStr = Field(description="Payment market")
    __properties: ClassVar[List[str]] = [
        "id",
        "product_id",
        "date",
        "amount",
        "currency",
        "status",
        "service",
        "market",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UnifiedPaymentModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UnifiedPaymentModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "product_id": obj.get("product_id"),
                "date": obj.get("date"),
                "amount": obj.get("amount"),
                "currency": obj.get("currency"),
                "status": obj.get("status"),
                "service": obj.get("service"),
                "market": obj.get("market"),
            }
        )
        return _obj



# coding: utf-8

"""
    Payment API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Union
from crypticorn.pay.client.models.now_payment_status import NowPaymentStatus
from typing import Optional, Set
from typing_extensions import Self


class NowPaymentModel(BaseModel):
    """
    NowPaymentModel
    """  # noqa: E501

    id: StrictStr = Field(description="Product ID")
    payment_id: StrictInt = Field(description="Unique payment identifier")
    invoice_id: StrictInt = Field(description="Associated invoice ID")
    product_id: StrictStr = Field(description="Product ID")
    user_id: StrictStr = Field(description="User ID")
    order_id: StrictStr = Field(description="Internal order ID")
    paid_amount: Union[StrictFloat, StrictInt] = Field(
        description="Actually paid amount"
    )
    pay_amount: Union[StrictFloat, StrictInt] = Field(description="Amount to pay")
    pay_currency: StrictStr = Field(description="Payment currency")
    status: NowPaymentStatus = Field(description="Current payment status")
    updated_at: StrictInt = Field(
        description="Payment last update timestamp in milliseconds"
    )
    __properties: ClassVar[List[str]] = [
        "id",
        "payment_id",
        "invoice_id",
        "product_id",
        "user_id",
        "order_id",
        "paid_amount",
        "pay_amount",
        "pay_currency",
        "status",
        "updated_at",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of NowPaymentModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of NowPaymentModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "payment_id": obj.get("payment_id"),
                "invoice_id": obj.get("invoice_id"),
                "product_id": obj.get("product_id"),
                "user_id": obj.get("user_id"),
                "order_id": obj.get("order_id"),
                "paid_amount": obj.get("paid_amount"),
                "pay_amount": obj.get("pay_amount"),
                "pay_currency": obj.get("pay_currency"),
                "status": obj.get("status"),
                "updated_at": obj.get("updated_at"),
            }
        )
        return _obj



# coding: utf-8

"""
    Payment API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self


class Currency(BaseModel):
    """
    Model representing a single cryptocurrency supported by the API.  This model contains detailed information about each cryptocurrency available for payments, including its identifiers, network details, validation patterns, and configuration settings.
    """  # noqa: E501

    id: StrictInt = Field(description="Unique identifier for the currency")
    code: StrictStr = Field(description="Currency code/ticker symbol")
    name: StrictStr = Field(description="Full name of the currency")
    enable: StrictBool = Field(description="Whether the currency is currently enabled")
    wallet_regex: StrictStr = Field(
        description="Regex pattern for valid wallet addresses"
    )
    priority: StrictInt = Field(description="Priority ranking of the currency")
    extra_id_exists: StrictBool = Field(
        description="Whether the currency requires an extra ID/memo/tag"
    )
    extra_id_regex: Optional[StrictStr] = None
    logo_url: StrictStr = Field(description="URL path to currency logo image")
    track: StrictBool = Field(description="Whether the currency is being tracked")
    cg_id: Optional[StrictStr] = None
    is_maxlimit: StrictBool = Field(
        description="Whether the currency has a maximum limit"
    )
    network: Optional[StrictStr] = None
    smart_contract: Optional[StrictStr] = None
    network_precision: Optional[StrictInt] = None
    __properties: ClassVar[List[str]] = [
        "id",
        "code",
        "name",
        "enable",
        "wallet_regex",
        "priority",
        "extra_id_exists",
        "extra_id_regex",
        "logo_url",
        "track",
        "cg_id",
        "is_maxlimit",
        "network",
        "smart_contract",
        "network_precision",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Currency from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if extra_id_regex (nullable) is None
        # and model_fields_set contains the field
        if self.extra_id_regex is None and "extra_id_regex" in self.model_fields_set:
            _dict["extra_id_regex"] = None

        # set to None if cg_id (nullable) is None
        # and model_fields_set contains the field
        if self.cg_id is None and "cg_id" in self.model_fields_set:
            _dict["cg_id"] = None

        # set to None if network (nullable) is None
        # and model_fields_set contains the field
        if self.network is None and "network" in self.model_fields_set:
            _dict["network"] = None

        # set to None if smart_contract (nullable) is None
        # and model_fields_set contains the field
        if self.smart_contract is None and "smart_contract" in self.model_fields_set:
            _dict["smart_contract"] = None

        # set to None if network_precision (nullable) is None
        # and model_fields_set contains the field
        if (
            self.network_precision is None
            and "network_precision" in self.model_fields_set
        ):
            _dict["network_precision"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Currency from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "code": obj.get("code"),
                "name": obj.get("name"),
                "enable": obj.get("enable"),
                "wallet_regex": obj.get("wallet_regex"),
                "priority": obj.get("priority"),
                "extra_id_exists": obj.get("extra_id_exists"),
                "extra_id_regex": obj.get("extra_id_regex"),
                "logo_url": obj.get("logo_url"),
                "track": obj.get("track"),
                "cg_id": obj.get("cg_id"),
                "is_maxlimit": obj.get("is_maxlimit"),
                "network": obj.get("network"),
                "smart_contract": obj.get("smart_contract"),
                "network_precision": obj.get("network_precision"),
            }
        )
        return _obj



from crypticorn.trade import (
    ApiClient,
    APIKeysApi,
    BotsApi,
    Configuration,
    ExchangesApi,
    FuturesTradingPanelApi,
    NotificationsApi,
    OrdersApi,
    StatusApi,
    StrategiesApi,
    TradingActionsApi,
)
from crypticorn.common import BaseURL, ApiVersion, Service, apikey_header as aph


class TradeClient:
    """
    A client for interacting with the Crypticorn Trade API.
    """

    def __init__(
        self,
        base_url: BaseURL,
        api_version: ApiVersion,
        api_key: str = None,
        jwt: str = None,
    ):
        self.host = f"{base_url.value}/{api_version.value}/{Service.TRADE.value}"
        self.config = Configuration(
            host=self.host,
            access_token=jwt,
            api_key={aph.scheme_name: api_key} if api_key else None,
            api_key_prefix=({aph.scheme_name: aph.model.name} if api_key else None),
        )
        self.base_client = ApiClient(configuration=self.config)
        # Instantiate all the endpoint clients
        self.bots = BotsApi(self.base_client)
        self.exchanges = ExchangesApi(self.base_client)
        self.notifications = NotificationsApi(self.base_client)
        self.orders = OrdersApi(self.base_client)
        self.status = StatusApi(self.base_client)
        self.strategies = StrategiesApi(self.base_client)
        self.actions = TradingActionsApi(self.base_client)
        self.futures = FuturesTradingPanelApi(self.base_client)
        self.api_keys = APIKeysApi(self.base_client)



# coding: utf-8

# flake8: noqa

"""
    Klines Service API

         API for retrieving OHLCV data, funding rates, and symbol information from Binance.          ## WebSocket Support     Connect to `/ws` to receive real-time OHLCV updates.          Example subscription message:     ```json     {         \"action\": \"subscribe\",         \"market\": \"spot\",         \"symbol\": \"BTCUSDT\",         \"timeframe\": \"15m\"     }     ```     

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from crypticorn.klines.client.api.funding_rates_api import FundingRatesApi
from crypticorn.klines.client.api.health_check_api import HealthCheckApi
from crypticorn.klines.client.api.ohlcv_data_api import OHLCVDataApi
from crypticorn.klines.client.api.symbols_api import SymbolsApi
from crypticorn.klines.client.api.udf_api import UDFApi

# import ApiClient
from crypticorn.klines.client.api_response import ApiResponse
from crypticorn.klines.client.api_client import ApiClient
from crypticorn.klines.client.configuration import Configuration
from crypticorn.klines.client.exceptions import OpenApiException
from crypticorn.klines.client.exceptions import ApiTypeError
from crypticorn.klines.client.exceptions import ApiValueError
from crypticorn.klines.client.exceptions import ApiKeyError
from crypticorn.klines.client.exceptions import ApiAttributeError
from crypticorn.klines.client.exceptions import ApiException

# import models into sdk package
from crypticorn.klines.client.models.base_response_health_check_response import (
    BaseResponseHealthCheckResponse,
)
from crypticorn.klines.client.models.base_response_list_funding_rate_response import (
    BaseResponseListFundingRateResponse,
)
from crypticorn.klines.client.models.base_response_list_str import BaseResponseListStr
from crypticorn.klines.client.models.base_response_ohlcv_response import (
    BaseResponseOHLCVResponse,
)
from crypticorn.klines.client.models.error_response import ErrorResponse
from crypticorn.klines.client.models.exchange import Exchange
from crypticorn.klines.client.models.funding_rate_response import FundingRateResponse
from crypticorn.klines.client.models.http_validation_error import HTTPValidationError
from crypticorn.klines.client.models.health_check_response import HealthCheckResponse
from crypticorn.klines.client.models.history_error_response import HistoryErrorResponse
from crypticorn.klines.client.models.history_no_data_response import (
    HistoryNoDataResponse,
)
from crypticorn.klines.client.models.history_success_response import (
    HistorySuccessResponse,
)
from crypticorn.klines.client.models.market import Market
from crypticorn.klines.client.models.ohlcv_response import OHLCVResponse
from crypticorn.klines.client.models.resolution import Resolution
from crypticorn.klines.client.models.response_get_history_udf_history_get import (
    ResponseGetHistoryUdfHistoryGet,
)
from crypticorn.klines.client.models.search_symbol_response import SearchSymbolResponse
from crypticorn.klines.client.models.sort_direction import SortDirection
from crypticorn.klines.client.models.symbol_group_response import SymbolGroupResponse
from crypticorn.klines.client.models.symbol_info_response import SymbolInfoResponse
from crypticorn.klines.client.models.symbol_type import SymbolType
from crypticorn.klines.client.models.timeframe import Timeframe
from crypticorn.klines.client.models.udf_config_response import UDFConfigResponse
from crypticorn.klines.client.models.validation_error import ValidationError
from crypticorn.klines.client.models.validation_error_loc_inner import (
    ValidationErrorLocInner,
)



from enum import Enum


class ApiErrorType(str, Enum):
    """Type of API error"""

    USER_ERROR = "user error"
    """user error by people using our services"""
    EXCHANGE_ERROR = "exchange error"
    """re-tryable error by the exchange or network conditions"""
    SERVER_ERROR = "server error"
    """server error that needs a new version rollout for a fix"""
    NO_ERROR = "no error"
    """error that does not need to be handled or does not affect the program or is a placeholder."""


class ApiErrorIdentifier(str, Enum):
    """API error identifiers"""

    SUCCESS = "success"
    UNAUTHORIZED = "invalid_api_key"
    INVALID_SIGNATURE = "invalid_signature"
    INVALID_TIMESTAMP = "invalid_timestamp"
    IP_RESTRICTED = "ip_address_is_not_authorized"
    PERMISSION_DENIED = "insufficient_permissions_spot_and_futures_required"
    USER_FROZEN = "user_account_is_frozen"
    RATE_LIMIT = "rate_limit_exceeded"
    INVALID_PARAMETER = "invalid_parameter_provided"
    REQUEST_SCOPE_EXCEEDED = "request_scope_limit_exceeded"
    CONTENT_TYPE_ERROR = "invalid_content_type"
    URL_NOT_FOUND = "requested_resource_not_found"
    ORDER_NOT_FOUND = "order_does_not_exist"
    ORDER_ALREADY_FILLED = "order_is_already_filled"
    ORDER_IN_PROCESS = "order_is_being_processed"
    ORDER_LIMIT_EXCEEDED = "order_quantity_limit_exceeded"
    ORDER_PRICE_INVALID = "order_price_is_invalid"
    POST_ONLY_REJECTED = "post_only_order_would_immediately_match"
    SYMBOL_NOT_FOUND = "symbol_does_not_exist"
    CLIENT_ORDER_ID_REPEATED = "client_order_id_already_exists"

    POSITION_NOT_FOUND = "position_does_not_exist"
    POSITION_LIMIT_EXCEEDED = "position_limit_exceeded"
    POSITION_SUSPENDED = "position_opening_temporarily_suspended"
    INSUFFICIENT_BALANCE = "insufficient_balance"
    INSUFFICIENT_MARGIN = "insufficient_margin"

    LEVERAGE_EXCEEDED = "leverage_limit_exceeded"
    RISK_LIMIT_EXCEEDED = "risk_limit_exceeded"
    LIQUIDATION_PRICE_VIOLATION = "order_violates_liquidation_price_constraints"
    INVALID_MARGIN_MODE = "invalid_margin_mode"

    SYSTEM_ERROR = "internal_system_error"
    SYSTEM_CONFIG_ERROR = "system_configuration_error"
    SERVICE_UNAVAILABLE = "service_temporarily_unavailable"
    SYSTEM_BUSY = "system_is_busy_please_try_again_later"
    MAINTENANCE = "system_under_maintenance"
    RPC_TIMEOUT = "rpc_timeout"
    SETTLEMENT_IN_PROGRESS = "system_settlement_in_process"

    TRADING_SUSPENDED = "trading_is_suspended"
    TRADING_LOCKED = "trading_has_been_locked"

    UNKNOWN_ERROR = "unknown_error_occurred"
    HTTP_ERROR = "http_request_error"
    BLACK_SWAN = "black_swan"

    # our bot errors
    ACTION_EXPIRED = "trading_action_expired"
    ACTION_SKIPPED = "trading_action_skipped"
    BOT_DISABLED = "bot_disabled"
    ORDER_SIZE_TOO_SMALL = "order_size_too_small"
    ORDER_SIZE_TOO_LARGE = "order_size_too_large"
    HEDGE_MODE_NOT_ACTIVE = "hedge_mode_not_active"

    # our API errors
    API_KEY_ALREADY_EXISTS = "api_key_already_exists"
    DELETE_BOT_ERROR = "delete_bot_error"
    JWT_EXPIRED = "jwt_expired"
    BOT_STOPPING_COMPLETED = "bot_stopping_completed"
    OBJECT_NOT_FOUND = "object_not_found"
    STRATEGY_DISABLED = "strategy_disabled"
    API_KEY_IN_USE = "api_key_in_use_by_bots"
    BOT_ALREADY_DELETED = "bot_already_deleted"

    # our auth errors
    INVALID_API_KEY = "invalid_api_key"
    INVALID_BEARER = "invalid_bearer"
    NO_CREDENTIALS = "no_credentials"
    INSUFFICIENT_SCOPES = "insufficient_scopes"


class ApiErrorLevel(str, Enum):
    ERROR = "error"
    SUCCESS = "success"
    INFO = "info"
    WARNING = "warning"


class ApiError(Enum):
    """API error codes"""

    # Success
    SUCCESS = (ApiErrorIdentifier.SUCCESS, ApiErrorType.NO_ERROR, ApiErrorLevel.SUCCESS)
    # Authentication/Authorization
    UNAUTHORIZED = (
        ApiErrorIdentifier.UNAUTHORIZED,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    INVALID_SIGNATURE = (
        ApiErrorIdentifier.INVALID_SIGNATURE,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    INVALID_TIMESTAMP = (
        ApiErrorIdentifier.INVALID_TIMESTAMP,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    IP_RESTRICTED = (
        ApiErrorIdentifier.IP_RESTRICTED,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    PERMISSION_DENIED = (
        ApiErrorIdentifier.PERMISSION_DENIED,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )
    USER_FROZEN = (
        ApiErrorIdentifier.USER_FROZEN,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )

    # Rate Limiting
    RATE_LIMIT = (
        ApiErrorIdentifier.RATE_LIMIT,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )

    # Invalid Parameters
    INVALID_PARAMETER = (
        ApiErrorIdentifier.INVALID_PARAMETER,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    REQUEST_SCOPE_EXCEEDED = (
        ApiErrorIdentifier.REQUEST_SCOPE_EXCEEDED,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )
    CONTENT_TYPE_ERROR = (
        ApiErrorIdentifier.CONTENT_TYPE_ERROR,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    URL_NOT_FOUND = (
        ApiErrorIdentifier.URL_NOT_FOUND,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    SYMBOL_NOT_FOUND = (
        ApiErrorIdentifier.SYMBOL_NOT_FOUND,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )

    # Order Related
    ORDER_NOT_FOUND = (
        ApiErrorIdentifier.ORDER_NOT_FOUND,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    ORDER_ALREADY_FILLED = (
        ApiErrorIdentifier.ORDER_ALREADY_FILLED,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.INFO,
    )
    ORDER_IN_PROCESS = (
        ApiErrorIdentifier.ORDER_IN_PROCESS,
        ApiErrorType.NO_ERROR,
        ApiErrorLevel.INFO,
    )
    ORDER_LIMIT_EXCEEDED = (
        ApiErrorIdentifier.ORDER_LIMIT_EXCEEDED,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )
    ORDER_PRICE_INVALID = (
        ApiErrorIdentifier.ORDER_PRICE_INVALID,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    POST_ONLY_REJECTED = (
        ApiErrorIdentifier.POST_ONLY_REJECTED,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    BLACK_SWAN = (
        ApiErrorIdentifier.BLACK_SWAN,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.INFO,
    )
    CLIENT_ORDER_ID_REPEATED = (
        ApiErrorIdentifier.CLIENT_ORDER_ID_REPEATED,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    # CONTRA_ORDER_MISSING = "No contra order available"

    # Position Related
    POSITION_NOT_FOUND = (
        ApiErrorIdentifier.POSITION_NOT_FOUND,
        ApiErrorType.NO_ERROR,
        ApiErrorLevel.INFO,
    )  # not an error because user can close a position manually which we don't know about
    POSITION_LIMIT_EXCEEDED = (
        ApiErrorIdentifier.POSITION_LIMIT_EXCEEDED,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )
    POSITION_SUSPENDED = (
        ApiErrorIdentifier.POSITION_SUSPENDED,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )
    # FUTURES_BRAWL_RESTRICTED = "Operation restricted for Futures Brawl", ApiErrorType.USER_ERROR

    # Balance/Margin
    INSUFFICIENT_BALANCE = (
        ApiErrorIdentifier.INSUFFICIENT_BALANCE,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )
    INSUFFICIENT_MARGIN = (
        ApiErrorIdentifier.INSUFFICIENT_MARGIN,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )

    # Leverage/Risk
    LEVERAGE_EXCEEDED = (
        ApiErrorIdentifier.LEVERAGE_EXCEEDED,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    RISK_LIMIT_EXCEEDED = (
        ApiErrorIdentifier.RISK_LIMIT_EXCEEDED,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    LIQUIDATION_PRICE_VIOLATION = (
        ApiErrorIdentifier.LIQUIDATION_PRICE_VIOLATION,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    INVALID_MARGIN_MODE = (
        ApiErrorIdentifier.INVALID_MARGIN_MODE,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )

    # System Status
    SYSTEM_ERROR = (
        ApiErrorIdentifier.SYSTEM_ERROR,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )
    SYSTEM_CONFIG_ERROR = (
        ApiErrorIdentifier.SYSTEM_CONFIG_ERROR,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )
    SERVICE_UNAVAILABLE = (
        ApiErrorIdentifier.SERVICE_UNAVAILABLE,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )
    SYSTEM_BUSY = (
        ApiErrorIdentifier.SYSTEM_BUSY,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )
    MAINTENANCE = (
        ApiErrorIdentifier.MAINTENANCE,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )
    RPC_TIMEOUT = (
        ApiErrorIdentifier.RPC_TIMEOUT,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )
    SETTLEMENT_IN_PROGRESS = (
        ApiErrorIdentifier.SETTLEMENT_IN_PROGRESS,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )

    # Trading Status
    TRADING_SUSPENDED = (
        ApiErrorIdentifier.TRADING_SUSPENDED,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )
    TRADING_LOCKED = (
        ApiErrorIdentifier.TRADING_LOCKED,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )

    # Generic
    UNKNOWN_ERROR = (
        ApiErrorIdentifier.UNKNOWN_ERROR,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )
    HTTP_ERROR = (
        ApiErrorIdentifier.HTTP_ERROR,
        ApiErrorType.EXCHANGE_ERROR,
        ApiErrorLevel.ERROR,
    )

    # Our Bot Errors
    ACTION_EXPIRED = (
        ApiErrorIdentifier.ACTION_EXPIRED,
        ApiErrorType.NO_ERROR,
        ApiErrorLevel.INFO,
    )
    ACTION_SKIPPED = (
        ApiErrorIdentifier.ACTION_SKIPPED,
        ApiErrorType.NO_ERROR,
        ApiErrorLevel.INFO,
    )
    BOT_DISABLED = (
        ApiErrorIdentifier.BOT_DISABLED,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.WARNING,
    )
    ORDER_SIZE_TOO_SMALL = (
        ApiErrorIdentifier.ORDER_SIZE_TOO_SMALL,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.WARNING,
    )
    ORDER_SIZE_TOO_LARGE = (
        ApiErrorIdentifier.ORDER_SIZE_TOO_LARGE,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.WARNING,
    )
    HEDGE_MODE_NOT_ACTIVE = (
        ApiErrorIdentifier.HEDGE_MODE_NOT_ACTIVE,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )

    # our API errors
    API_KEY_ALREADY_EXISTS = (
        ApiErrorIdentifier.API_KEY_ALREADY_EXISTS,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )
    DELETE_BOT_ERROR = (
        ApiErrorIdentifier.DELETE_BOT_ERROR,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    JWT_EXPIRED = (
        ApiErrorIdentifier.JWT_EXPIRED,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    BOT_STOPPING_COMPLETED = (
        ApiErrorIdentifier.BOT_STOPPING_COMPLETED,
        ApiErrorType.NO_ERROR,
        ApiErrorLevel.INFO,
    )
    OBJECT_NOT_FOUND = (
        ApiErrorIdentifier.OBJECT_NOT_FOUND,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    STRATEGY_DISABLED = (
        ApiErrorIdentifier.STRATEGY_DISABLED,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    API_KEY_IN_USE = (
        ApiErrorIdentifier.API_KEY_IN_USE,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )
    BOT_ALREADY_DELETED = (
        ApiErrorIdentifier.BOT_ALREADY_DELETED,
        ApiErrorType.SERVER_ERROR,
        ApiErrorLevel.ERROR,
    )

    # our auth errors
    INVALID_API_KEY = (
        ApiErrorIdentifier.INVALID_API_KEY,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )
    INVALID_BEARER = (
        ApiErrorIdentifier.INVALID_BEARER,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )
    NO_CREDENTIALS = (
        ApiErrorIdentifier.NO_CREDENTIALS,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )
    INSUFFICIENT_SCOPES = (
        ApiErrorIdentifier.INSUFFICIENT_SCOPES,
        ApiErrorType.USER_ERROR,
        ApiErrorLevel.ERROR,
    )

    @property
    def identifier(self) -> str:
        return self.value[0]

    @property
    def type(self) -> ApiErrorType:
        return self.value[1]

    @property
    def level(self) -> ApiErrorLevel:
        return self.value[2]



# coding: utf-8

"""
    Payment API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from crypticorn.pay.client.models.now_fee_structure import NowFeeStructure
from crypticorn.pay.client.models.now_payment_status import NowPaymentStatus
from typing import Optional, Set
from typing_extensions import Self


class NowWebhookPayload(BaseModel):
    """
    Model for NOWPayments webhook (IPN) payload.
    """  # noqa: E501

    actually_paid: Union[StrictFloat, StrictInt] = Field(
        description="Actually paid amount"
    )
    actually_paid_at_fiat: Union[StrictFloat, StrictInt] = Field(
        description="Actually paid amount in fiat currency"
    )
    fee: NowFeeStructure = Field(description="Fee structure for the payment")
    invoice_id: StrictInt = Field(description="Associated invoice ID")
    order_description: StrictStr = Field(description="Order description")
    order_id: StrictStr = Field(description="Internal order ID")
    outcome_amount: Union[StrictFloat, StrictInt] = Field(description="Outcome amount")
    outcome_currency: StrictStr = Field(description="Outcome currency")
    parent_payment_id: Optional[StrictInt] = None
    pay_address: StrictStr = Field(description="Payment destination address")
    pay_amount: Union[StrictFloat, StrictInt] = Field(description="Amount to pay")
    pay_currency: StrictStr = Field(description="Payment currency")
    payment_id: StrictInt = Field(description="Unique payment identifier")
    payment_status: NowPaymentStatus = Field(description="Current payment status")
    price_amount: Union[StrictFloat, StrictInt] = Field(
        description="Original price amount"
    )
    price_currency: StrictStr = Field(description="Original price currency")
    purchase_id: StrictStr = Field(description="Purchase ID")
    updated_at: StrictInt = Field(
        description="Payment last update timestamp in milliseconds"
    )
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "actually_paid",
        "actually_paid_at_fiat",
        "fee",
        "invoice_id",
        "order_description",
        "order_id",
        "outcome_amount",
        "outcome_currency",
        "parent_payment_id",
        "pay_address",
        "pay_amount",
        "pay_currency",
        "payment_id",
        "payment_status",
        "price_amount",
        "price_currency",
        "purchase_id",
        "updated_at",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of NowWebhookPayload from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set(
            [
                "additional_properties",
            ]
        )

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of fee
        if self.fee:
            _dict["fee"] = self.fee.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if parent_payment_id (nullable) is None
        # and model_fields_set contains the field
        if (
            self.parent_payment_id is None
            and "parent_payment_id" in self.model_fields_set
        ):
            _dict["parent_payment_id"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of NowWebhookPayload from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "actually_paid": obj.get("actually_paid"),
                "actually_paid_at_fiat": obj.get("actually_paid_at_fiat"),
                "fee": (
                    NowFeeStructure.from_dict(obj["fee"])
                    if obj.get("fee") is not None
                    else None
                ),
                "invoice_id": obj.get("invoice_id"),
                "order_description": obj.get("order_description"),
                "order_id": obj.get("order_id"),
                "outcome_amount": obj.get("outcome_amount"),
                "outcome_currency": obj.get("outcome_currency"),
                "parent_payment_id": obj.get("parent_payment_id"),
                "pay_address": obj.get("pay_address"),
                "pay_amount": obj.get("pay_amount"),
                "pay_currency": obj.get("pay_currency"),
                "payment_id": obj.get("payment_id"),
                "payment_status": obj.get("payment_status"),
                "price_amount": obj.get("price_amount"),
                "price_currency": obj.get("price_currency"),
                "purchase_id": obj.get("purchase_id"),
                "updated_at": obj.get("updated_at"),
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



from enum import Enum


class Domain(Enum):
    PROD = "crypticorn.com"
    DEV = "crypticorn.dev"


class BaseURL(Enum):
    PROD = "https://api.crypticorn.com"
    DEV = "https://api.crypticorn.dev"
    LOCAL = "http://localhost"
    DOCKER = "http://host.docker.internal"


class ApiVersion(Enum):
    V1 = "v1"


class Service(Enum):
    HIVE = "hive"
    KLINES = "klines"
    PAY = "pay"
    TRADE = "trade"
    AUTH = "auth"



from crypticorn.auth import (
    ApiClient,
    Configuration,
    AdminApi,
    ServiceApi,
    UserApi,
    WalletApi,
    AuthApi,
)
from crypticorn.common import BaseURL, ApiVersion, Service, apikey_header as aph


class AuthClient:
    """
    A client for interacting with the Crypticorn Auth API.
    """

    def __init__(
        self,
        base_url: BaseURL,
        api_version: ApiVersion,
        api_key: str = None,
        jwt: str = None,
    ):
        self.host = f"{base_url.value}/{api_version.value}/{Service.AUTH.value}"
        self.config = Configuration(
            host=self.host,
            access_token=jwt,
            api_key={aph.scheme_name: api_key} if api_key else None,
            api_key_prefix=({aph.scheme_name: aph.model.name} if api_key else None),
            debug=True,
        )
        self.base_client = ApiClient(configuration=self.config)
        # Instantiate all the endpoint clients
        self.admin = AdminApi(self.base_client)
        self.service = ServiceApi(self.base_client)
        self.user = UserApi(self.base_client)
        self.wallet = WalletApi(self.base_client)
        self.login = AuthApi(self.base_client)

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.base_client.close()



# coding: utf-8

"""
    Hive AI API

    API for Hive AI model training and evaluation

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List
from crypticorn.hive.client.models.coins import Coins
from crypticorn.hive.client.models.evaluation import Evaluation
from crypticorn.hive.client.models.model_status import ModelStatus
from crypticorn.hive.client.models.target import Target
from crypticorn.hive.client.models.target_type import TargetType
from typing import Optional, Set
from typing_extensions import Self


class Model(BaseModel):
    """
    Pydantic model for model response
    """  # noqa: E501

    model_id: StrictInt = Field(description="Unique model identifier")
    name: StrictStr = Field(description="Model name")
    coin_id: Coins = Field(description="Coin ID")
    target: Target = Field(description="Target variable")
    status: ModelStatus = Field(description="Model status")
    target_type: TargetType = Field(description="Target type")
    evaluations: List[Evaluation] = Field(description="Model evaluations")
    user_id: StrictStr = Field(description="Developer user ID")
    created_at: datetime = Field(description="Model creation timestamp")
    updated_at: datetime = Field(description="Model update timestamp")
    __properties: ClassVar[List[str]] = [
        "model_id",
        "name",
        "coin_id",
        "target",
        "status",
        "target_type",
        "evaluations",
        "user_id",
        "created_at",
        "updated_at",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Model from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in evaluations (list)
        _items = []
        if self.evaluations:
            for _item_evaluations in self.evaluations:
                if _item_evaluations:
                    _items.append(_item_evaluations.to_dict())
            _dict["evaluations"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Model from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "model_id": obj.get("model_id"),
                "name": obj.get("name"),
                "coin_id": obj.get("coin_id"),
                "target": obj.get("target"),
                "status": obj.get("status"),
                "target_type": obj.get("target_type"),
                "evaluations": (
                    [Evaluation.from_dict(_item) for _item in obj["evaluations"]]
                    if obj.get("evaluations") is not None
                    else None
                ),
                "user_id": obj.get("user_id"),
                "created_at": obj.get("created_at"),
                "updated_at": obj.get("updated_at"),
            }
        )
        return _obj



from fastapi import Depends, HTTPException, Query, status
from fastapi.security import HTTPAuthorizationCredentials
from typing_extensions import Annotated, Doc
import json

from crypticorn.auth import AuthClient, Verify200Response
from crypticorn.auth.client.exceptions import ApiException
from crypticorn.common import (
    ApiError,
    Scope,
    ApiVersion,
    BaseURL,
    Domain,
    apikey_header,
    http_bearer,
)


class AuthHandler:
    """
    Middleware for verifying API requests. Verifies the validity of the authentication token, allowed scopes, etc.
    """

    def __init__(
        self,
        base_url: Annotated[
            BaseURL, Doc("The base URL for the auth service.")
        ] = BaseURL.PROD,
        api_version: Annotated[
            ApiVersion, Doc("The API version of the auth service.")
        ] = ApiVersion.V1,
        whitelist: Annotated[
            list[Domain],
            Doc(
                "The domains of which requests are allowed full access to the service."
            ),
        ] = [
            Domain.PROD,
            Domain.DEV,
        ],  # TODO: decide whether this is needed, else omit
    ):
        self.whitelist = whitelist
        self.auth_client = AuthClient(base_url=base_url, api_version=api_version)

        self.no_credentials_exception = HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail=ApiError.NO_CREDENTIALS.identifier,
        )

    async def _verify_api_key(self, api_key: str) -> None:
        """
        Verifies the API key.
        """
        return await self.auth_client.login.verify_api_key(api_key)

    async def _verify_bearer(
        self, bearer: HTTPAuthorizationCredentials
    ) -> Verify200Response:
        """
        Verifies the bearer token.
        """
        self.auth_client.config.access_token = bearer.credentials
        return await self.auth_client.login.verify()

    async def _validate_scopes(
        self, api_scopes: list[Scope], user_scopes: list[Scope]
    ) -> bool:
        """
        Checks if the user scopes are a subset of the API scopes.
        """
        if not set(api_scopes).issubset(user_scopes):
            raise HTTPException(
                status_code=status.HTTP_403_FORBIDDEN,
                detail=ApiError.INSUFFICIENT_SCOPES.identifier,
            )

    async def _extract_message(self, e: ApiException) -> str:
        """
        Tries to extract the message from the body of the exception.
        """
        try:
            load = json.loads(e.body)
        except (json.JSONDecodeError, TypeError):
            return e.body
        else:
            common_keys = ["message"]
            for key in common_keys:
                if key in load:
                    return load[key]
            return load

    async def _handle_exception(self, e: Exception) -> HTTPException:
        """
        Handles exceptions and returns a HTTPException with the appropriate status code and detail.
        """
        if isinstance(e, ApiException):
            return HTTPException(
                status_code=e.status,
                detail=await self._extract_message(e),
            )
        elif isinstance(e, HTTPException):
            return e
        else:
            return HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=str(e),
            )

    async def api_key_auth(
        self,
        api_key: Annotated[str | None, Depends(apikey_header)] = None,
        scopes: list[Scope] = [],
    ) -> Verify200Response:
        """
        Verifies the API key and checks if the user scopes are a subset of the API scopes.
        Use this function if you only want to allow access via the API key.
        """
        return await self.combined_auth(bearer=None, api_key=api_key, scopes=scopes)

    async def bearer_auth(
        self,
        bearer: Annotated[
            HTTPAuthorizationCredentials | None,
            Depends(http_bearer),
        ] = None,
        scopes: list[Scope] = [],
    ) -> Verify200Response:
        """
        Verifies the bearer token and checks if the user scopes are a subset of the API scopes.
        Use this function if you only want to allow access via the bearer token.
        """
        return await self.combined_auth(bearer=bearer, api_key=None, scopes=scopes)

    async def combined_auth(
        self,
        bearer: Annotated[
            HTTPAuthorizationCredentials | None, Depends(http_bearer)
        ] = None,
        api_key: Annotated[str | None, Depends(apikey_header)] = None,
        scopes: list[Scope] = [],
    ) -> Verify200Response:
        """
        Verifies the bearer token and/or API key and checks if the user scopes are a subset of the API scopes.
        Returns early on the first successful verification, otherwise tries all available tokens.
        Use this function if you want to allow access via either the bearer token or the API key.
        """
        tokens = [bearer, api_key]

        last_error = None
        for token in tokens:
            try:
                if token is None:
                    continue
                res = None
                if isinstance(token, str):
                    res = await self._verify_api_key(token)
                elif isinstance(token, HTTPAuthorizationCredentials):
                    res = await self._verify_bearer(token)
                if res is None:
                    continue
                if scopes:
                    await self._validate_scopes(
                        scopes, [Scope.from_str(scope) for scope in res.scopes]
                    )
                return res

            except Exception as e:
                last_error = await self._handle_exception(e)
                continue

        raise last_error or self.no_credentials_exception

    async def ws_api_key_auth(
        self,
        api_key: Annotated[str | None, Query()] = None,
        scopes: list[Scope] = [],
    ) -> Verify200Response:
        """
        Verifies the API key and checks if the user scopes are a subset of the API scopes.
        Use this function if you only want to allow access via the API key.
        """
        return await self.api_key_auth(api_key=api_key, scopes=scopes)

    async def ws_bearer_auth(
        self,
        bearer: Annotated[str | None, Query()] = None,
        scopes: list[Scope] = [],
    ) -> Verify200Response:
        """
        Verifies the bearer token and checks if the user scopes are a subset of the API scopes.
        Use this function if you only want to allow access via the bearer token.
        """
        credentials = (
            HTTPAuthorizationCredentials(scheme="Bearer", credentials=bearer)
            if bearer
            else None
        )
        return await self.bearer_auth(bearer=credentials, scopes=scopes)

    async def ws_combined_auth(
        self,
        bearer: Annotated[str | None, Query()] = None,
        api_key: Annotated[str | None, Query()] = None,
        scopes: list[Scope] = [],
    ) -> Verify200Response:
        """
        Verifies the bearer token and/or API key and checks if the user scopes are a subset of the API scopes.
        Use this function if you want to allow access via either the bearer token or the API key.
        """
        credentials = (
            HTTPAuthorizationCredentials(scheme="Bearer", credentials=bearer)
            if bearer
            else None
        )
        return await self.combined_auth(
            bearer=credentials, api_key=api_key, scopes=scopes
        )



# coding: utf-8

"""
    Payment API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set
from typing_extensions import Self


class Payment(BaseModel):
    """
    Model representing a single payment.
    """  # noqa: E501

    payment_id: StrictInt = Field(description="Unique payment identifier")
    invoice_id: Optional[StrictInt] = None
    payment_status: StrictStr = Field(description="Current payment status")
    pay_address: StrictStr = Field(description="Payment destination address")
    payin_extra_id: Optional[StrictStr] = None
    price_amount: Union[StrictFloat, StrictInt] = Field(
        description="Original price amount"
    )
    price_currency: StrictStr = Field(description="Original price currency")
    pay_amount: Union[StrictFloat, StrictInt] = Field(description="Amount to pay")
    actually_paid: Union[StrictFloat, StrictInt] = Field(
        description="Actually paid amount"
    )
    pay_currency: StrictStr = Field(description="Payment currency")
    order_id: Optional[StrictStr] = None
    order_description: Optional[StrictStr] = None
    purchase_id: Optional[StrictInt] = None
    outcome_amount: Optional[Union[StrictFloat, StrictInt]] = None
    outcome_currency: Optional[StrictStr] = None
    payout_hash: Optional[StrictStr] = None
    payin_hash: Optional[StrictStr] = None
    created_at: StrictStr = Field(description="Payment creation timestamp")
    updated_at: StrictStr = Field(description="Payment last update timestamp")
    type: StrictStr = Field(description="Type of payment (e.g., crypto2crypto)")
    payment_extra_ids: Optional[List[StrictInt]] = None
    parent_payment_id: Optional[StrictInt] = None
    origin_type: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "payment_id",
        "invoice_id",
        "payment_status",
        "pay_address",
        "payin_extra_id",
        "price_amount",
        "price_currency",
        "pay_amount",
        "actually_paid",
        "pay_currency",
        "order_id",
        "order_description",
        "purchase_id",
        "outcome_amount",
        "outcome_currency",
        "payout_hash",
        "payin_hash",
        "created_at",
        "updated_at",
        "type",
        "payment_extra_ids",
        "parent_payment_id",
        "origin_type",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Payment from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if invoice_id (nullable) is None
        # and model_fields_set contains the field
        if self.invoice_id is None and "invoice_id" in self.model_fields_set:
            _dict["invoice_id"] = None

        # set to None if payin_extra_id (nullable) is None
        # and model_fields_set contains the field
        if self.payin_extra_id is None and "payin_extra_id" in self.model_fields_set:
            _dict["payin_extra_id"] = None

        # set to None if order_id (nullable) is None
        # and model_fields_set contains the field
        if self.order_id is None and "order_id" in self.model_fields_set:
            _dict["order_id"] = None

        # set to None if order_description (nullable) is None
        # and model_fields_set contains the field
        if (
            self.order_description is None
            and "order_description" in self.model_fields_set
        ):
            _dict["order_description"] = None

        # set to None if purchase_id (nullable) is None
        # and model_fields_set contains the field
        if self.purchase_id is None and "purchase_id" in self.model_fields_set:
            _dict["purchase_id"] = None

        # set to None if outcome_amount (nullable) is None
        # and model_fields_set contains the field
        if self.outcome_amount is None and "outcome_amount" in self.model_fields_set:
            _dict["outcome_amount"] = None

        # set to None if outcome_currency (nullable) is None
        # and model_fields_set contains the field
        if (
            self.outcome_currency is None
            and "outcome_currency" in self.model_fields_set
        ):
            _dict["outcome_currency"] = None

        # set to None if payout_hash (nullable) is None
        # and model_fields_set contains the field
        if self.payout_hash is None and "payout_hash" in self.model_fields_set:
            _dict["payout_hash"] = None

        # set to None if payin_hash (nullable) is None
        # and model_fields_set contains the field
        if self.payin_hash is None and "payin_hash" in self.model_fields_set:
            _dict["payin_hash"] = None

        # set to None if payment_extra_ids (nullable) is None
        # and model_fields_set contains the field
        if (
            self.payment_extra_ids is None
            and "payment_extra_ids" in self.model_fields_set
        ):
            _dict["payment_extra_ids"] = None

        # set to None if parent_payment_id (nullable) is None
        # and model_fields_set contains the field
        if (
            self.parent_payment_id is None
            and "parent_payment_id" in self.model_fields_set
        ):
            _dict["parent_payment_id"] = None

        # set to None if origin_type (nullable) is None
        # and model_fields_set contains the field
        if self.origin_type is None and "origin_type" in self.model_fields_set:
            _dict["origin_type"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Payment from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "payment_id": obj.get("payment_id"),
                "invoice_id": obj.get("invoice_id"),
                "payment_status": obj.get("payment_status"),
                "pay_address": obj.get("pay_address"),
                "payin_extra_id": obj.get("payin_extra_id"),
                "price_amount": obj.get("price_amount"),
                "price_currency": obj.get("price_currency"),
                "pay_amount": obj.get("pay_amount"),
                "actually_paid": obj.get("actually_paid"),
                "pay_currency": obj.get("pay_currency"),
                "order_id": obj.get("order_id"),
                "order_description": obj.get("order_description"),
                "purchase_id": obj.get("purchase_id"),
                "outcome_amount": obj.get("outcome_amount"),
                "outcome_currency": obj.get("outcome_currency"),
                "payout_hash": obj.get("payout_hash"),
                "payin_hash": obj.get("payin_hash"),
                "created_at": obj.get("created_at"),
                "updated_at": obj.get("updated_at"),
                "type": obj.get("type"),
                "payment_extra_ids": obj.get("payment_extra_ids"),
                "parent_payment_id": obj.get("parent_payment_id"),
                "origin_type": obj.get("origin_type"),
            }
        )
        return _obj



import asyncio
import os

import dotenv
import pytest
import pytest_asyncio
from crypticorn.common import AuthHandler, BaseURL, Scope, http_bearer
from fastapi import HTTPException
from fastapi.security import HTTPAuthorizationCredentials

dotenv.load_dotenv()

# JWT
EXPIRED_JWT = os.getenv("EXPIRED_JWT")
VALID_JWT = os.getenv("VALID_JWT")

# API KEY
FULL_SCOPE_API_KEY = os.getenv("FULL_SCOPE_API_KEY")
ONE_SCOPE_API_KEY = os.getenv("ONE_SCOPE_API_KEY")
EXPIRED_API_KEY = os.getenv("EXPIRED_API_KEY")

# ASSERT SCOPE
ALL_SCOPES = [scope.value for scope in Scope]
JWT_SCOPE = Scope.READ_PREDICTIONS
API_KEY_SCOPE = Scope.READ_TRADE_BOTS

# ERROR MESSAGES
API_KEY_EXPIRED = "API key expired"
API_KEY_INVALID = "Invalid API key"
BEARER_EXPIRED = "jwt expired"

# Debug
UPDATE_SCOPES = "you probably need to bring the scopes in both the api client and the auth service in sync"

# Each function is tested without credentials, with invalid credentials, and with valid credentials.
# The test is successful if the correct HTTPException is raised.
#


@pytest_asyncio.fixture
async def auth_handler() -> AuthHandler:
    return AuthHandler(BaseURL.LOCAL)


# COMBINED AUTH


@pytest.mark.asyncio
async def test_combined_auth_without_credentials(auth_handler: AuthHandler):
    """Without credentials"""
    with pytest.raises(HTTPException) as e:
        await auth_handler.combined_auth(bearer=None, api_key=None)
    assert e.value.status_code == 401
    assert e.value.detail == auth_handler.no_credentials_exception.detail


# BEARER
@pytest.mark.asyncio
async def test_combined_auth_with_invalid_bearer_token(auth_handler: AuthHandler):
    """With invalid bearer token"""
    with pytest.raises(HTTPException) as e:
        await auth_handler.combined_auth(
            bearer=HTTPAuthorizationCredentials(scheme="Bearer", credentials="123"),
            api_key=None,
        )
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_combined_auth_with_expired_bearer_token(auth_handler: AuthHandler):
    """With expired bearer token"""
    with pytest.raises(HTTPException) as e:
        await auth_handler.combined_auth(
            bearer=HTTPAuthorizationCredentials(
                scheme="Bearer", credentials=EXPIRED_JWT
            ),
            api_key=None,
        )
    assert e.value.status_code == 401
    assert e.value.detail == BEARER_EXPIRED


@pytest.mark.asyncio
async def test_combined_auth_with_valid_bearer_token(auth_handler: AuthHandler):
    """With valid bearer token"""
    res = await auth_handler.combined_auth(
        bearer=HTTPAuthorizationCredentials(scheme="Bearer", credentials=VALID_JWT),
        api_key=None,
    )
    assert JWT_SCOPE in res.scopes, UPDATE_SCOPES


# API KEY
@pytest.mark.asyncio
async def test_combined_auth_with_invalid_api_key(auth_handler: AuthHandler):
    """With invalid api key"""
    with pytest.raises(HTTPException) as e:
        return await auth_handler.combined_auth(bearer=None, api_key="123")
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_combined_auth_with_full_scope_valid_api_key(auth_handler: AuthHandler):
    """With full scope valid api key"""
    res = await auth_handler.combined_auth(bearer=None, api_key=FULL_SCOPE_API_KEY)
    assert res.scopes == ALL_SCOPES, UPDATE_SCOPES


@pytest.mark.asyncio
async def test_combined_auth_with_one_scope_valid_api_key(auth_handler: AuthHandler):
    """With one scope valid api key"""
    res = await auth_handler.combined_auth(bearer=None, api_key=ONE_SCOPE_API_KEY)
    assert API_KEY_SCOPE in res.scopes, UPDATE_SCOPES


@pytest.mark.asyncio
async def test_combined_auth_with_expired_api_key(auth_handler: AuthHandler):
    """With expired api key"""
    with pytest.raises(HTTPException) as e:
        res = await auth_handler.combined_auth(bearer=None, api_key=EXPIRED_API_KEY)
    assert e.value.status_code == 401
    assert e.value.detail == API_KEY_EXPIRED


# API KEY AUTH
@pytest.mark.asyncio
async def test_api_key_auth_without_api_key(auth_handler: AuthHandler):
    """Without api key"""
    with pytest.raises(HTTPException) as e:
        return await auth_handler.api_key_auth(api_key=None)
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_api_key_auth_with_invalid_api_key(auth_handler: AuthHandler):
    """With invalid api key"""
    with pytest.raises(HTTPException) as e:
        return await auth_handler.api_key_auth(api_key="123")
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_api_key_auth_with_full_scope_valid_api_key(auth_handler: AuthHandler):
    """With full scope valid api key"""
    res = await auth_handler.api_key_auth(api_key=FULL_SCOPE_API_KEY)
    assert res.scopes == ALL_SCOPES, UPDATE_SCOPES


@pytest.mark.asyncio
async def test_api_key_auth_with_one_scope_valid_api_key(auth_handler: AuthHandler):
    """With one scope valid api key"""
    res = await auth_handler.api_key_auth(api_key=ONE_SCOPE_API_KEY)
    assert API_KEY_SCOPE in res.scopes, UPDATE_SCOPES


@pytest.mark.asyncio
async def test_api_key_auth_with_expired_api_key(auth_handler: AuthHandler):
    """With expired api key"""
    with pytest.raises(HTTPException) as e:
        res = await auth_handler.api_key_auth(api_key=EXPIRED_API_KEY)
    assert e.value.status_code == 401
    assert e.value.detail == API_KEY_EXPIRED


# BEARER AUTH
@pytest.mark.asyncio
async def test_bearer_auth_without_bearer(auth_handler: AuthHandler):
    """Without bearer"""
    with pytest.raises(HTTPException) as e:
        return await auth_handler.bearer_auth(bearer=None)
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_bearer_auth_with_invalid_bearer(auth_handler: AuthHandler):
    """With invalid bearer"""
    with pytest.raises(HTTPException) as e:
        return await auth_handler.bearer_auth(
            bearer=HTTPAuthorizationCredentials(scheme="Bearer", credentials="123")
        )
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_bearer_auth_with_valid_bearer(auth_handler: AuthHandler):
    """With valid bearer"""
    res = await auth_handler.bearer_auth(
        bearer=HTTPAuthorizationCredentials(scheme="Bearer", credentials=VALID_JWT)
    )
    assert JWT_SCOPE in res.scopes, UPDATE_SCOPES


@pytest.mark.asyncio
async def test_bearer_auth_with_expired_bearer(auth_handler: AuthHandler):
    """With expired bearer"""
    with pytest.raises(HTTPException) as e:
        return await auth_handler.bearer_auth(
            bearer=HTTPAuthorizationCredentials(
                scheme="Bearer", credentials=EXPIRED_JWT
            )
        )
    assert e.value.status_code == 401
    assert e.value.detail == BEARER_EXPIRED


# WS COMBINED AUTH
@pytest.mark.asyncio
async def test_ws_combined_auth_without_credentials(auth_handler: AuthHandler):
    """Without credentials"""
    with pytest.raises(HTTPException) as e:
        return await auth_handler.ws_combined_auth(bearer=None, api_key=None)
    assert e.value.status_code == 401
    assert e.value.detail == auth_handler.no_credentials_exception.detail


# BEARER
@pytest.mark.asyncio
async def test_ws_combined_auth_with_invalid_bearer(auth_handler: AuthHandler):
    """With invalid bearer"""
    with pytest.raises(HTTPException) as e:
        return await auth_handler.ws_combined_auth(bearer="123", api_key=None)
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_ws_combined_auth_with_valid_bearer(auth_handler: AuthHandler):
    res = await auth_handler.ws_combined_auth(bearer=VALID_JWT, api_key=None)
    assert JWT_SCOPE in res.scopes, UPDATE_SCOPES


@pytest.mark.asyncio
async def test_ws_combined_auth_with_expired_bearer(auth_handler: AuthHandler):
    """With expired bearer"""
    with pytest.raises(HTTPException) as e:
        res = await auth_handler.ws_combined_auth(bearer=EXPIRED_JWT, api_key=None)
    assert e.value.status_code == 401
    assert e.value.detail == BEARER_EXPIRED


# API KEY
@pytest.mark.asyncio
async def test_ws_combined_auth_with_invalid_api_key(auth_handler: AuthHandler):
    """With invalid api key"""
    with pytest.raises(HTTPException) as e:
        res = await auth_handler.ws_combined_auth(bearer=None, api_key="123")
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_ws_combined_auth_with_full_scope_valid_api_key(
    auth_handler: AuthHandler,
):
    """With full scope valid api key"""
    res = await auth_handler.ws_combined_auth(bearer=None, api_key=FULL_SCOPE_API_KEY)
    assert res.scopes == ALL_SCOPES, UPDATE_SCOPES


@pytest.mark.asyncio
async def test_ws_combined_auth_with_one_scope_valid_api_key(auth_handler: AuthHandler):
    """With one scope valid api key"""
    res = await auth_handler.ws_combined_auth(bearer=None, api_key=ONE_SCOPE_API_KEY)
    assert API_KEY_SCOPE in res.scopes, UPDATE_SCOPES


@pytest.mark.asyncio
async def test_ws_combined_auth_with_expired_api_key(auth_handler: AuthHandler):
    """With expired api key"""
    with pytest.raises(HTTPException) as e:
        res = await auth_handler.ws_combined_auth(bearer=None, api_key=EXPIRED_API_KEY)
    assert e.value.status_code == 401
    assert e.value.detail == API_KEY_EXPIRED


# WS BEARER AUTH
@pytest.mark.asyncio
async def test_ws_bearer_auth_without_bearer(auth_handler: AuthHandler):
    with pytest.raises(HTTPException) as e:
        return await auth_handler.ws_bearer_auth(bearer=None)
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_ws_bearer_auth_with_invalid_bearer(auth_handler: AuthHandler):
    with pytest.raises(HTTPException) as e:
        return await auth_handler.ws_bearer_auth(bearer="123")
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_ws_bearer_auth_with_valid_bearer(auth_handler: AuthHandler):
    res = await auth_handler.ws_bearer_auth(bearer=VALID_JWT)
    assert JWT_SCOPE in res.scopes, UPDATE_SCOPES


# WS API KEY AUTH
@pytest.mark.asyncio
async def test_ws_api_key_auth_without_api_key(auth_handler: AuthHandler):
    with pytest.raises(HTTPException) as e:
        return await auth_handler.ws_api_key_auth(api_key=None)
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_ws_api_key_auth_with_invalid_api_key(auth_handler: AuthHandler):
    with pytest.raises(HTTPException) as e:
        return await auth_handler.ws_api_key_auth(api_key="123")
    assert e.value.status_code == 401


@pytest.mark.asyncio
async def test_ws_api_key_auth_with_expired_api_key(auth_handler: AuthHandler):
    with pytest.raises(HTTPException) as e:
        return await auth_handler.ws_api_key_auth(api_key=EXPIRED_API_KEY)
    assert e.value.status_code == 401
    assert e.value.detail == API_KEY_EXPIRED


@pytest.mark.asyncio
async def test_ws_api_key_auth_with_full_scope_valid_api_key(auth_handler: AuthHandler):
    res = await auth_handler.ws_api_key_auth(api_key=FULL_SCOPE_API_KEY)
    assert res.scopes == ALL_SCOPES, UPDATE_SCOPES


@pytest.mark.asyncio
async def test_ws_api_key_auth_with_one_scope_valid_api_key(auth_handler: AuthHandler):
    res = await auth_handler.ws_api_key_auth(api_key=ONE_SCOPE_API_KEY)
    assert API_KEY_SCOPE in res.scopes, UPDATE_SCOPES


# print(asyncio.run(test_ws_api_key_auth_with_one_scope_valid_api_key(AuthHandler(BaseURL.LOCAL))))



from crypticorn.hive import HiveClient
from crypticorn.klines import KlinesClient
from crypticorn.pay import PayClient
from crypticorn.trade import TradeClient
from crypticorn.common import BaseURL, ApiVersion


class ApiClient:
    """
    The official client for interacting with the Crypticorn API.

    It is consisting of multiple microservices covering the whole stack of the Crypticorn project.
    """

    def __init__(
        self,
        base_url: BaseURL = BaseURL.PROD,
        api_key: str = None,
        jwt: str = None,
        hive_version: ApiVersion = ApiVersion.V1,
        klines_version: ApiVersion = ApiVersion.V1,
        pay_version: ApiVersion = ApiVersion.V1,
        trade_version: ApiVersion = ApiVersion.V1,
        auth_version: ApiVersion = ApiVersion.V1,
    ):
        self.base_url = base_url
        self.api_key = api_key
        self.jwt = jwt
        self.hive = HiveClient(base_url, hive_version, api_key, jwt)
        self.trade = TradeClient(base_url, trade_version, api_key, jwt)
        self.klines = KlinesClient(base_url, klines_version, api_key, jwt)
        self.pay = PayClient(base_url, pay_version, api_key, jwt)
        # currently not working due to circular import since the AUTH_Handler
        # is also using the ApiClient
        # self.auth = AuthClient(base_url, auth_version, api_key, jwt)

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()

    async def close(self):
        """Close all client sessions."""
        clients = [
            self.hive.base_client,
            self.trade.base_client,
            self.klines.base_client,
            self.pay.base_client,
        ]

        for client in clients:
            if hasattr(client, "close"):
                await client.close()



# coding: utf-8

"""
    Klines Service API

         API for retrieving OHLCV data, funding rates, and symbol information from Binance.          ## WebSocket Support     Connect to `/ws` to receive real-time OHLCV updates.          Example subscription message:     ```json     {         \"action\": \"subscribe\",         \"market\": \"spot\",         \"symbol\": \"BTCUSDT\",         \"timeframe\": \"15m\"     }     ```     

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class Resolution(str, Enum):
    """
    Resolution
    """

    """
    allowed enum values
    """
    ENUM_15 = "15"
    ENUM_30 = "30"
    ENUM_60 = "60"
    ENUM_240 = "240"
    ENUM_1D = "1D"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Resolution from a JSON string"""
        return cls(json.loads(json_str))



# coding: utf-8

"""
    Klines Service API

         API for retrieving OHLCV data, funding rates, and symbol information from Binance.          ## WebSocket Support     Connect to `/ws` to receive real-time OHLCV updates.          Example subscription message:     ```json     {         \"action\": \"subscribe\",         \"market\": \"spot\",         \"symbol\": \"BTCUSDT\",         \"timeframe\": \"15m\"     }     ```     

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set
from typing_extensions import Self


class HistoryNoDataResponse(BaseModel):
    """
    HistoryNoDataResponse
    """  # noqa: E501

    s: Optional[StrictStr] = "no_data"
    t: Optional[List[StrictInt]] = None
    o: Optional[List[Union[StrictFloat, StrictInt]]] = None
    h: Optional[List[Union[StrictFloat, StrictInt]]] = None
    l: Optional[List[Union[StrictFloat, StrictInt]]] = None
    c: Optional[List[Union[StrictFloat, StrictInt]]] = None
    v: Optional[List[Union[StrictFloat, StrictInt]]] = None
    __properties: ClassVar[List[str]] = ["s", "t", "o", "h", "l", "c", "v"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of HistoryNoDataResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of HistoryNoDataResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "s": obj.get("s") if obj.get("s") is not None else "no_data",
                "t": obj.get("t"),
                "o": obj.get("o"),
                "h": obj.get("h"),
                "l": obj.get("l"),
                "c": obj.get("c"),
                "v": obj.get("v"),
            }
        )
        return _obj



# coding: utf-8

# flake8: noqa
"""
    Payment API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from crypticorn.pay.client.models.http_validation_error import HTTPValidationError
from crypticorn.pay.client.models.now_api_status_res import NowAPIStatusRes
from crypticorn.pay.client.models.now_create_invoice_req import NowCreateInvoiceReq
from crypticorn.pay.client.models.now_create_invoice_res import NowCreateInvoiceRes
from crypticorn.pay.client.models.now_fee_structure import NowFeeStructure
from crypticorn.pay.client.models.now_payment_status import NowPaymentStatus
from crypticorn.pay.client.models.now_webhook_payload import NowWebhookPayload
from crypticorn.pay.client.models.payment_status import PaymentStatus
from crypticorn.pay.client.models.product_model import ProductModel
from crypticorn.pay.client.models.product_subs_model import ProductSubsModel
from crypticorn.pay.client.models.services import Services
from crypticorn.pay.client.models.unified_payment_model import UnifiedPaymentModel
from crypticorn.pay.client.models.validation_error import ValidationError
from crypticorn.pay.client.models.validation_error_loc_inner import (
    ValidationErrorLocInner,
)



# coding: utf-8

# flake8: noqa
"""
    Klines Service API

         API for retrieving OHLCV data, funding rates, and symbol information from Binance.          ## WebSocket Support     Connect to `/ws` to receive real-time OHLCV updates.          Example subscription message:     ```json     {         \"action\": \"subscribe\",         \"market\": \"spot\",         \"symbol\": \"BTCUSDT\",         \"timeframe\": \"15m\"     }     ```     

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from crypticorn.klines.client.models.base_response_health_check_response import (
    BaseResponseHealthCheckResponse,
)
from crypticorn.klines.client.models.base_response_list_funding_rate_response import (
    BaseResponseListFundingRateResponse,
)
from crypticorn.klines.client.models.base_response_list_str import BaseResponseListStr
from crypticorn.klines.client.models.base_response_ohlcv_response import (
    BaseResponseOHLCVResponse,
)
from crypticorn.klines.client.models.error_response import ErrorResponse
from crypticorn.klines.client.models.exchange import Exchange
from crypticorn.klines.client.models.funding_rate_response import FundingRateResponse
from crypticorn.klines.client.models.http_validation_error import HTTPValidationError
from crypticorn.klines.client.models.health_check_response import HealthCheckResponse
from crypticorn.klines.client.models.history_error_response import HistoryErrorResponse
from crypticorn.klines.client.models.history_no_data_response import (
    HistoryNoDataResponse,
)
from crypticorn.klines.client.models.history_success_response import (
    HistorySuccessResponse,
)
from crypticorn.klines.client.models.market import Market
from crypticorn.klines.client.models.ohlcv_response import OHLCVResponse
from crypticorn.klines.client.models.resolution import Resolution
from crypticorn.klines.client.models.response_get_history_udf_history_get import (
    ResponseGetHistoryUdfHistoryGet,
)
from crypticorn.klines.client.models.search_symbol_response import SearchSymbolResponse
from crypticorn.klines.client.models.sort_direction import SortDirection
from crypticorn.klines.client.models.symbol_group_response import SymbolGroupResponse
from crypticorn.klines.client.models.symbol_info_response import SymbolInfoResponse
from crypticorn.klines.client.models.symbol_type import SymbolType
from crypticorn.klines.client.models.timeframe import Timeframe
from crypticorn.klines.client.models.udf_config_response import UDFConfigResponse
from crypticorn.klines.client.models.validation_error import ValidationError
from crypticorn.klines.client.models.validation_error_loc_inner import (
    ValidationErrorLocInner,
)



# coding: utf-8

# flake8: noqa

"""
    Payment API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from crypticorn.pay.client.api.now_payments_api import NOWPaymentsApi
from crypticorn.pay.client.api.payments_api import PaymentsApi
from crypticorn.pay.client.api.products_api import ProductsApi
from crypticorn.pay.client.api.status_api import StatusApi

# import ApiClient
from crypticorn.pay.client.api_response import ApiResponse
from crypticorn.pay.client.api_client import ApiClient
from crypticorn.pay.client.configuration import Configuration
from crypticorn.pay.client.exceptions import OpenApiException
from crypticorn.pay.client.exceptions import ApiTypeError
from crypticorn.pay.client.exceptions import ApiValueError
from crypticorn.pay.client.exceptions import ApiKeyError
from crypticorn.pay.client.exceptions import ApiAttributeError
from crypticorn.pay.client.exceptions import ApiException

# import models into sdk package
from crypticorn.pay.client.models.http_validation_error import HTTPValidationError
from crypticorn.pay.client.models.now_api_status_res import NowAPIStatusRes
from crypticorn.pay.client.models.now_create_invoice_req import NowCreateInvoiceReq
from crypticorn.pay.client.models.now_create_invoice_res import NowCreateInvoiceRes
from crypticorn.pay.client.models.now_fee_structure import NowFeeStructure
from crypticorn.pay.client.models.now_payment_status import NowPaymentStatus
from crypticorn.pay.client.models.now_webhook_payload import NowWebhookPayload
from crypticorn.pay.client.models.payment_status import PaymentStatus
from crypticorn.pay.client.models.product_model import ProductModel
from crypticorn.pay.client.models.product_subs_model import ProductSubsModel
from crypticorn.pay.client.models.services import Services
from crypticorn.pay.client.models.unified_payment_model import UnifiedPaymentModel
from crypticorn.pay.client.models.validation_error import ValidationError
from crypticorn.pay.client.models.validation_error_loc_inner import (
    ValidationErrorLocInner,
)
